library easy_duration;

/// Return Duration base on int input
///
/// ex: 1.seconds -> Duration(seconds: 1);
extension EasyDurationInt on int {
  /// Returns current int in Duration of seconds
  Duration get seconds {
    return Duration(seconds: this);
  }

  /// Short-hand for [seconds]
  Duration get s {
    return Duration(seconds: this);
  }

  /// Return current int in Duration of miliseconds
  Duration get miliseconds {
    return Duration(milliseconds: this);
  }

  /// Short-hand for miliseconds
  Duration get ms {
    return Duration(milliseconds: this);
  }

  /// Return curretn int in  Duration of microseconds
  Duration get microseconds {
    return Duration(microseconds: this);
  }

  /// Short-hand for microseconds
  Duration get us {
    return Duration(microseconds: this);
  }

  /// Return current int in Duration of minutes
  Duration get minutes {
    return Duration(minutes: this);
  }

  /// Short-hand for minutes
  Duration get m {
    return Duration(minutes: this);
  }

  /// Return current int in Duration of hours
  Duration get hours {
    return Duration(hours: this);
  }

  /// Short-hand for hours
  Duration get h {
    return Duration(hours: this);
  }

  /// Return current int in Duration of days
  Duration get days {
    return Duration(days: this);
  }

  /// Short-hand for days
  Duration get d {
    return Duration(days: this);
  }

  /// Return current int in Duration of month (30 days / month)
  Duration get months {
    return Duration(days: 30 * this);
  }

  /// Short-hand for months
  Duration get M {
    return Duration(days: 30 * this);
  }

  /// Return current int in Duration of years (356 days/ year)
  Duration get years {
    return Duration(days: 365 * this);
  }

  /// Short-hand for years
  Duration get y {
    return Duration(days: 365 * this);
  }

  /// Return current int in Duration of weeks (7 days/ week)
  Duration get weeks {
    return Duration(days: 7 * this);
  }

  /// Short-hand for weeks
  Duration get w {
    return Duration(days: 7 * this);
  }

  /// Return current int in Duration of circles (You decide circle length)
  ///
  /// ex: 2.circles(2.s) means -> 2 cicles of 2s = 4s
  Duration circles(Duration circleLength) {
    final inUs = circleLength.inMicroseconds;
    return (this * inUs).us;
  }
}

extension EasyDurationDouble on double {
  /// Returns current int in Duration of seconds
  ///
  /// It will be convert to ms first
  ///
  /// 1.5s --> 1500ms
  Duration get seconds {
    final ms = (this * 1000).floor();
    return Duration(milliseconds: ms);
  }

  /// Short-hand for [seconds]
  Duration get s {
    final ms = (this * 1000).floor();
    return Duration(milliseconds: ms);
  }

  /// Return current int in Duration of miliseconds
  Duration get miliseconds {
    final us = (this * 1000).floor();
    return Duration(microseconds: us);
  }

  /// Short-hand for miliseconds
  Duration get ms {
    final us = (this * 1000).floor();
    return Duration(microseconds: us);
  }

  /// Return curretn int in  Duration of microseconds
  ///
  /// This will be truncate()
  ///
  /// ex: 1.2 ms --> 1 ms
  ///
  /// 1.7ms --> 1ms
  Duration get microseconds {
    final us = this.toInt();
    return Duration(microseconds: us);
  }

  /// Short-hand for microseconds
  Duration get us {
    final us = this.toInt();
    return Duration(microseconds: us);
  }

  /// Return current int in Duration of minutes
  ///
  /// It will be convert to second the truncate()
  ///
  /// 1.5m -> 90s = 90s
  ///
  /// 1.2m -> 72s = 72s
  ///
  /// 1.02m -> 61.2s = 61s
  Duration get minutes {
    final secs = (this * 60).toInt();
    return Duration(seconds: secs);
  }

  /// Short-hand for minutes
  Duration get m {
    final secs = (this * 60).toInt();
    return Duration(seconds: secs);
  }

  /// Return current int in Duration of hours
  Duration get hours {
    final mins = (this * 60).toInt();
    return Duration(minutes: mins);
  }

  /// Short-hand for hours
  Duration get h {
    final mins = (this * 60).toInt();
    return Duration(minutes: mins);
  }

  /// Return current int in Duration of days
  Duration get days {
    final hours = (this * 24).toInt();
    return Duration(hours: hours);
  }

  /// Short-hand for days
  Duration get d {
    final hours = (this * 24).toInt();
    return Duration(hours: hours);
  }

  /// Return current int in Duration of month (30 days / month)
  Duration get months {
    final days = (this * 30).toInt();
    return Duration(days: days);
  }

  /// Short-hand for months
  Duration get M {
    final days = (this * 30).toInt();
    return Duration(days: days);
  }

  /// Return current int in Duration of years (356 days/ year)
  Duration get years {
    return Duration(days: (365 * this).toInt());
  }

  /// Short-hand for years
  Duration get y {
    return Duration(days: (365 * this).toInt());
  }

  /// Return current int in Duration of weeks (7 days/ week)
  Duration get weeks {
    return Duration(days: (7 * this).toInt());
  }

  /// Short-hand for weeks
  Duration get w {
    return Duration(days: (7 * this).toInt());
  }

  /// Return current int in Duration of circles (You decide circle length)
  ///
  /// ex: 2.circles(2.s) means -> 2 cicles of 2s = 4s
  Duration circles(Duration circleLength) {
    final inUs = circleLength.inMicroseconds;
    return (this * inUs).us;
  }
}

/// Return the day after [currentDate]
DateTime theDayAfter(DateTime currentDate) {
  return currentDate.add(1.d);
}

/// Return the day before [currentDate]
DateTime theDayBefore(DateTime currentDate) {
  return currentDate.subtract(1.d);
}
